@@ -18,7 +18,6 @@
 import datetime
 
 from oslo_log import log as logging
-import six
 from webob import exc
 
 from nova.api.openstack import api_version_request
@@ -43,6 +42,7 @@ def _get_context(req):
 
 class AggregateController(wsgi.Controller):
     """The Host Aggregates API controller for the OpenStack API."""
+
     def __init__(self):
         super(AggregateController, self).__init__()
         self.api = compute.AggregateAPI()
@@ -211,17 +211,15 @@ class AggregateController(wsgi.Controller):
                 exception.AggregateHostNotFound,
                 exception.ComputeHostNotFound) as e:
             LOG.error('Failed to remove host %s from aggregate %s. Error: %s',
-                      host, id, six.text_type(e))
+                      host, id, str(e))
             msg = _('Cannot remove host %(host)s in aggregate %(id)s') % {
                         'host': host, 'id': id}
             raise exc.HTTPNotFound(explanation=msg)
         except (exception.InvalidAggregateAction,
                 exception.ResourceProviderUpdateConflict) as e:
             LOG.error('Failed to remove host %s from aggregate %s. Error: %s',
-                      host, id, six.text_type(e))
-            msg = _('Cannot remove host %(host)s in aggregate %(id)s') % {
-                        'host': host, 'id': id}
-            raise exc.HTTPConflict(explanation=msg)
+                      host, id, str(e))
+            raise exc.HTTPConflict(explanation=e.format_message())
         return self._marshall_aggregate(req, aggregate)
 
     @wsgi.expected_errors((400, 404))
@@ -243,7 +241,8 @@ class AggregateController(wsgi.Controller):
                                                            id, metadata)
         except exception.AggregateNotFound as e:
             raise exc.HTTPNotFound(explanation=e.format_message())
-        except exception.InvalidAggregateAction as e:
+        except (exception.InvalidAggregateAction,
+                exception.AggregateMetadataKeyExists) as e:
             raise exc.HTTPBadRequest(explanation=e.format_message())
 
         return self._marshall_aggregate(req, aggregate)
@@ -289,7 +288,7 @@ class AggregateController(wsgi.Controller):
         for image_req in body.get('cache'):
             image_ids.append(image_req['id'])
 
-        if image_ids != list(set(image_ids)):
+        if sorted(image_ids) != sorted(list(set(image_ids))):
             raise exc.HTTPBadRequest(
                 explanation=_('Duplicate images in request'))
 
