@@ -161,7 +161,7 @@ class Forbidden(NovaException):
 
 class NotSupported(NovaException):
     # This exception use return code as 400 and can be used
-    # directly or as base exception for operations whihc are not
+    # directly or as base exception for operations which are not
     # supported in Nova. Any feature that is not yet implemented
     # but plan to implement in future (example: Cyborg
     # integration operations), should use this exception as base
@@ -465,6 +465,12 @@ class InvalidAggregateActionUpdateMeta(InvalidAggregateAction):
                 "%(aggregate_id)s. Reason: %(reason)s.")
 
 
+class AggregateMetadataKeyExists(NovaException):
+    msg_fmt = _("Aggregate %(aggregate_id)s already contain metadata "
+                "key %(key)s.")
+    code = 400
+
+
 class InvalidSortKey(Invalid):
     msg_fmt = _("Sort key supplied was not valid.")
 
@@ -700,6 +706,22 @@ class VolumeNotFound(NotFound):
     msg_fmt = _("Volume %(volume_id)s could not be found.")
 
 
+class ShareNotFound(NotFound):
+    msg_fmt = _("Share %(share_id)s could not be found.")
+
+
+class ShareUmountError(NovaException):
+    msg_fmt = _("Share id %(share_id)s umount error "
+                "from server %(server_id)s.\n"
+                "Reason: %(reason)s.")
+
+
+class ShareMountError(NovaException):
+    msg_fmt = _("Share id %(share_id)s mount error "
+                "from server %(server_id)s.\n"
+                "Reason: %(reason)s.")
+
+
 class VolumeTypeNotFound(NotFound):
     msg_fmt = _("Volume type %(id_or_name)s could not be found.")
 
@@ -1040,6 +1062,12 @@ class SecurityGroupCannotBeApplied(Invalid):
                 " in order to apply security groups.")
 
 
+class SecurityGroupConnectionStateConflict(Invalid):
+    code = 409
+    msg_fmt = _("Cannot apply both stateful and stateless security groups on "
+                "the same port.")
+
+
 class NoUniqueMatch(NovaException):
     msg_fmt = _("No Unique Match Found.")
     code = 409
@@ -2536,5 +2564,15 @@ class NotSupportedComputeForEvacuateV295(NotSupported):
                 "for compute %(expected), current version %(currently)s")
 
 
+class EphemeralEncryptionSecretNotFound(Invalid):
+    msg_fmt = _(
+        'Encryption secret %(secret_uuid)s was not found in the key manager')
+
+
+class EphemeralEncryptionCleanupFailed(NovaException):
+    msg_fmt = _("Failed to clean up ephemeral encryption secrets: "
+                "%(error)s")
+
+
 class HostConflict(Exception):
     pass
