@@ -56,12 +56,14 @@ import testtools
 from nova.api.openstack import wsgi_app
 from nova.compute import rpcapi as compute_rpcapi
 from nova import context
+import nova.crypto
 from nova.db.main import api as db_api
 from nova import exception
 from nova import objects
 from nova.objects import base as objects_base
 from nova import quota
 from nova.scheduler.client import report
+from nova.scheduler import utils as scheduler_utils
 from nova.tests import fixtures as nova_fixtures
 from nova.tests.unit import matchers
 from nova import utils
@@ -171,6 +173,12 @@ class TestCase(base.BaseTestCase):
     # base class when USES_DB is True.
     NUMBER_OF_CELLS = 1
 
+    # The stable compute id stuff is intentionally singleton-ish, which makes
+    # it a nightmare for testing multiple host/node combinations in tests like
+    # we do. So, mock it out by default, unless the test is specifically
+    # designed to handle it.
+    STUB_COMPUTE_ID = True
+
     def setUp(self):
         """Run before each test method to initialize test environment."""
         # Ensure BaseTestCase's ConfigureLogging fixture is disabled since
@@ -185,6 +193,7 @@ class TestCase(base.BaseTestCase):
         self._service_fixture_count = collections.defaultdict(int)
 
         self.useFixture(nova_fixtures.OpenStackSDKFixture())
+        self.useFixture(nova_fixtures.IsolatedGreenPoolFixture(self.id()))
 
         self.useFixture(log_fixture.get_logging_handle_error_fixture())
 
@@ -288,6 +297,10 @@ class TestCase(base.BaseTestCase):
         self.useFixture(nova_fixtures.GenericPoisonFixture())
         self.useFixture(nova_fixtures.SysFsPoisonFixture())
 
+        # Additional module names can be added to this set if needed
+        self.useFixture(nova_fixtures.ImportModulePoisonFixture(
+            set(['guestfs', 'libvirt'])))
+
         # make sure that the wsgi app is fully initialized for all testcase
         # instead of only once initialized for test worker
         wsgi_app.init_global_data.reset()
@@ -295,6 +308,27 @@ class TestCase(base.BaseTestCase):
         # Reset the placement client singleton
         report.PLACEMENTCLIENT = None
 
+        # Reset our local node uuid cache (and avoid writing to the
+        # local filesystem when we generate a new one).
+        if self.STUB_COMPUTE_ID:
+            self.useFixture(nova_fixtures.ComputeNodeIdFixture())
+
+        # Reset globals indicating affinity filter support. Some tests may set
+        # self.flags(enabled_filters=...) which could make the affinity filter
+        # support globals get set to a non-default configuration which affects
+        # all other tests.
+        scheduler_utils.reset_globals()
+
+        # Wait for bare greenlets spawn_n()'ed from a GreenThreadPoolExecutor
+        # to finish before moving on from the test. When greenlets from a
+        # previous test remain running, they may attempt to access structures
+        # (like the database) that have already been torn down and can cause
+        # the currently running test to fail.
+        self.useFixture(nova_fixtures.GreenThreadPoolShutdownWait())
+
+        # Reset the global key manager
+        nova.crypto._KEYMGR = None
+
     def _setup_cells(self):
         """Setup a normal cellsv2 environment.
 
@@ -388,6 +422,12 @@ class TestCase(base.BaseTestCase):
         for k, v in kw.items():
             CONF.set_override(k, v, group)
 
+    def reset_flags(self, *k, **kw):
+        """Reset flag variables for a test."""
+        group = kw.pop('group')
+        for flag in k:
+            CONF.clear_override(flag, group)
+
     def enforce_fk_constraints(self, engine=None):
         if engine is None:
             engine = db_api.get_engine()
@@ -453,7 +493,7 @@ class TestCase(base.BaseTestCase):
         self.computes[host] = compute
         return compute
 
-    def _run_periodics(self):
+    def _run_periodics(self, raise_on_error=False):
         """Run the update_available_resource task on every compute manager
 
         This runs periodics on the computes in an undefined order; some child
@@ -468,6 +508,15 @@ class TestCase(base.BaseTestCase):
             with context.target_cell(
                     ctx, self.host_mappings[host].cell_mapping) as cctxt:
                 compute.manager.update_available_resource(cctxt)
+
+        if raise_on_error:
+            if 'Traceback (most recent call last' in self.stdlog.logger.output:
+                # Get the last line of the traceback, for example:
+                #   TypeError: virNodeDeviceLookupByName() argument 2 must be
+                #   str or None, not Proxy
+                last_tb_line = self.stdlog.logger.output.splitlines()[-1]
+                raise TestingException(last_tb_line)
+
         LOG.info('Finished with periodics')
 
     def restart_compute_service(self, compute, keep_hypervisor_state=True):
@@ -683,6 +732,7 @@ class SubclassSignatureTestCase(testtools.TestCase, metaclass=abc.ABCMeta):
         raise NotImplementedError()
 
     def setUp(self):
+        self.useFixture(nova_fixtures.ConfFixture(CONF))
         self.base = self._get_base_class()
 
         super(SubclassSignatureTestCase, self).setUp()
@@ -695,16 +745,20 @@ class SubclassSignatureTestCase(testtools.TestCase, metaclass=abc.ABCMeta):
         # getmembers returns all members, including members inherited from
         # the base class. It's redundant for us to test these, but as
         # they'll always pass it's not worth the complexity to filter them out.
-        for (name, method) in inspect.getmembers(cls, inspect.ismethod):
+        for (name, method) in inspect.getmembers(cls, inspect.isfunction):
             # Subclass __init__ methods can usually be legitimately different
             if name == '__init__':
                 continue
 
+            # Skip subclass private functions
+            if name.startswith('_'):
+                continue
+
             while hasattr(method, '__wrapped__'):
                 # This is a wrapped function. The signature we're going to
                 # see here is that of the wrapper, which is almost certainly
                 # going to involve varargs and kwargs, and therefore is
-                # unlikely to be what we want. If the wrapper manupulates the
+                # unlikely to be what we want. If the wrapper manipulates the
                 # arguments taken by the wrapped function, the wrapped function
                 # isn't what we want either. In that case we're just stumped:
                 # if it ever comes up, add more knobs here to work round it (or
@@ -789,10 +843,10 @@ class MatchType(object):
         self.wanttype = wanttype
 
     def __eq__(self, other):
-        return type(other) == self.wanttype
+        return type(other) is self.wanttype
 
     def __ne__(self, other):
-        return type(other) != self.wanttype
+        return type(other) is not self.wanttype
 
     def __repr__(self):
         return "<MatchType:" + str(self.wanttype) + ">"
